Package: lumi
Type: Package
Title: BeadArray Specific Methods for Illumina Methylation and
        Expression Microarrays
Version: 2.62.0
Date: 2020-10-02
Depends: R (>= 2.10), Biobase (>= 2.5.5)
Imports: affy (>= 1.23.4), methylumi (>= 2.3.2), GenomicFeatures,
        GenomicRanges, annotate, lattice, mgcv (>= 1.4-0), nleqslv,
        KernSmooth, preprocessCore, RSQLite, DBI, AnnotationDbi, MASS,
        graphics, stats, stats4, methods
Suggests: beadarray, limma, vsn, lumiBarnes, lumiHumanAll.db,
        lumiHumanIDMapping, genefilter, RColorBrewer
Author: Pan Du, Richard Bourgon, Gang Feng, Simon Lin
Maintainer: Lei Huang <lhuang1998@gmail.com>
Description: The lumi package provides an integrated solution for the
    Illumina microarray data analysis. It includes functions of
    Illumina BeadStudio (GenomeStudio) data input, quality control,
    BeadArray-specific variance stabilization, normalization and gene
    annotation at the probe level. It also includes the functions of
    processing Illumina methylation microarrays, especially Illumina
    Infinium methylation microarrays.
License: LGPL (>= 2)
LazyLoad: yes
biocViews: Microarray, OneChannel, Preprocessing, DNAMethylation,
        QualityControl, TwoChannel
NeedsCompilation: no
Packaged: 2025-10-30 04:45:22 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/lumi
git_branch: RELEASE_3_22
git_last_commit: 1123e59
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
