import(methods)

importFrom(grDevices, colors)
importFrom(utils, head, packageDescription, read.table, tail,
           write.table)
importFrom(stats4, plot, summary)

importClassesFrom(
    Biobase, AnnotatedDataFrame, AssayData, eSet,
    ExpressionSet, Versioned, VersionedBiobase,
    Versions)
importMethodsFrom(
    Biobase, annotation, "annotation<-", assayData,
    "assayData<-", experimentData, "experimentData<-",
    featureData, "featureData<-", featureNames,
    "featureNames<-", pData, "pData<-", phenoData,
    "phenoData<-", exprs, "exprs<-", se.exprs,
    "se.exprs<-", sampleNames, "sampleNames<-",
    varLabels, varMetadata, coerce, combine, dim,
    initialize, notes, storageMode, show, "[", "[[",
    "$")
importFrom(
    Biobase, AnnotatedDataFrame, assayDataElement,
    "assayDataElement<-", assayDataElementNames,
    assayDataElementReplace, assayDataValidMembers,
    description, fData, "fData<-", "notes<-",
    protocolData, "protocolData<-", rowMin, "storageMode<-")
importClassesFrom(methylumi, MethyLumi, MethyLumiSet, MethyLumiQC, MethyLumiM)
importMethodsFrom(methylumi, methylated, unmethylated, "methylated<-", "unmethylated<-",
                 "betas", "betas<-", QCdata, combine, "[", 
 "controlData", "controlData<-", detection, "detection<-", "dataType", "dataType<-")
importFrom(methylumi, controlTypes, lumIDAT, methylumiR)

importFrom(mgcv, gam, smoothCon, mono.con, pcls, s, Predict.matrix)
importFrom(KernSmooth, locpoly)
importFrom(lattice, bwplot, panel.violin)
importFrom(nleqslv, nleqslv)
importFrom(annotate, lookUp, getSYMBOL)
importClassesFrom(RSQLite, SQLiteDriver, SQLiteConnection)
importMethodsFrom(RSQLite, dbReadTable, dbListTables, dbConnect, dbDisconnect, dbListFields)
importFrom(DBI, dbReadTable, dbListTables, dbConnect, dbDisconnect, dbListFields, dbDriver, 
           dbDataType, dbCommit, dbClearResult, SQLKeywords)

importFrom(AnnotationDbi, as.list, keys)
importClassesFrom(AnnotationDbi, AnnDbBimap)
importMethodsFrom(AnnotationDbi, "[", mappedkeys)
importFrom(graphics, pairs, boxplot, hist, image, lines, matplot, points, abline, axis, par,
           text, Axis, arrows, box, layout,  legend, rect, title)
importFrom(stats, density, heatmap, approx, as.dist, cmdscale, cor,
           density.default, dgamma, dist, ecdf, hclust, lm, median,
           pgamma, predict, quantile, sd, start, supsmu, var)
importFrom(MASS, rlm, psi.bisquare)
importFrom(affy, normalize.loess, mva.pairs, bg.adjust, plotDensity)
importFrom(preprocessCore, normalize.quantiles, normalize.quantiles.robust)

importFrom(GenomicFeatures, features)
importFrom(GenomicRanges, seqnames)


exportClasses(LumiBatch)
exportMethods(beadNum, "beadNum<-", boxplot, combine, density, detection, "detection<-", getHistory, hist, MAplot, pairs, plot, "sampleNames<-", exprs, "exprs<-", se.exprs, "se.exprs<-", show, summary, "[", coerce, "controlData", "controlData<-", asBigMatrix)

export(IlluminaID2nuID, addControlData2lumi, addNuID2lumi, addNuId2lumi,                                                            
bgAdjust, detectOutlier, detectionCall, estimateLumiCV, getChipInfo, getControlData, getControlProbe,
getControlType, getNuIDMappingInfo, id2seq, inverseVST, is.nuID, lumiB, lumiExpresso, lumiN, lumiQ,                       
lumiR, lumiR.batch, lumiT, nuID2EntrezID, nuID2IlluminaID, nuID2RefSeqID, nuID2probeID, nuID2targetID,
plotCDF, plotControlData, plotDensity, plotHousekeepingGene, plotSampleRelation, plotStringencyGene, plotVST, probeID2nuID,
produceGEOPlatformFile, produceGEOSampleInfoTemplate, produceGEOSubmissionFile, rsn, seq2id, ssn, targetID2nuID, vst, rankinvariant,
addAnnotationInfo, addControlData2methyLumiM, adjColorBias.quantile, adjColorBias.ssn, beta2m, bgAdjustMethylation, boxplotColorBias, colorBiasSummary, estimateBeta, estimateM,
estimateIntensity, estimateMethylationBG, importMethyIDAT, lumiMethyB, lumiMethyC, lumiMethyN, lumiMethyR, m2beta, normalizeMethylation.quantile, 
normalizeMethylation.ssn, plotColorBias2D, plotColorBias1D, produceMethylationGEOSubmissionFile, smoothQuantileNormalization, 
lumiMethyStatus, gammaFitEM, methylationCall, plotGammaFit, getChrInfo
)


